@@ -75,7 +75,7 @@ def compute_diff(e, eqs_dict, functions):
                 mid_expr = expr.OpExpr("^", e.exprs[0], expr.OpExpr("-", e.exprs[1], expr.AConst(1)))
                 return expr.OpExpr("*", e.exprs[1], expr.OpExpr("*", mid_expr, du)) 
             elif e.op == '/':
-                # d(u/v) = (u*dv + du * v)/ v^2
+                # d(u/v) = (du * v - u * dv)/ v^2   u'v-uv')/v²
                 # If v is constant, d(u/v) = 1/v * du, which is easier than using above rule.
                 if isinstance(e.exprs[1], expr.AConst) or \
                    isinstance(e.exprs[1], expr.FunExpr) and len(e.exprs == 0): # actually a constant
@@ -84,7 +84,7 @@ def compute_diff(e, eqs_dict, functions):
                 else:
                     du = rec(e.exprs[0])
                     dv = rec(e.exprs[1])
-                    numerator = expr.OpExpr("+", expr.OpExpr("*", e.exprs[0], dv), expr.OpExpr("*", du, e.exprs[1]))
+                    numerator = expr.OpExpr("-", expr.OpExpr("*", du, e.exprs[1]), expr.OpExpr("*", e.exprs[0], dv))
                     denominator = expr.OpExpr('*', e.exprs[1], e.exprs[1])
                     return expr.OpExpr('/', numerator, denominator)            
             else:
@@ -420,7 +420,8 @@ class CmdVerifier:
             # Split the subpre_expr if it is a conjunction, for example, A >= 0 && x >= A
             subpre_exprs = expr.split_conj(subpre_expr)
             for sub_expr in subpre_exprs:
-                if sub_expr.get_vars().isdisjoint(self.variable_names):
+                # vars is not empty and does not include variable names.
+                if sub_expr.get_vars() and sub_expr.get_vars().isdisjoint(self.variable_names):
                     self.infos[root_pos].assume.append(Condition(expr=sub_expr,
                                                                  origins=subpre.origins))
 
@@ -736,7 +737,7 @@ class CmdVerifier:
         elif isinstance(cur_hp, hcsp.Sequence):
             # Sequence of several commands, apply compute_wp from bottom to top
             cur_post = post
-            i_num = 0 # the number of IChoice and ITE in the sub-programs of Sequence.
+            
             for i in reversed(range(len(cur_hp.hps))):
                 sub_pos = (pos[0] + (i,), pos[1])
                 if sub_pos not in self.infos:
@@ -746,9 +747,6 @@ class CmdVerifier:
                 sub_info.assume += self.infos[pos].assume
                 sub_info.parent_type = cur_hp.type
 
-                if isinstance(get_pos(self.hp, sub_pos[0]), (hcsp.IChoice, hcsp.ITE)):
-                    i_num += 1
-
                 self.compute_wp(pos=sub_pos)
                 cur_post = sub_info.pre
             pre = cur_post
@@ -826,8 +824,9 @@ class CmdVerifier:
 
             # Currently assume out_hp is Skip.
             constraint = cur_hp.constraint
-            # The pre-condition computed for invariant and for dw rule is set None initially.
-            pre = None
+            # The pre-condition computed for invariant  is set as empty.
+            # The pre-condition for dw rule is set as None initially.
+            pre = []
             pre_dw = None
             ghost_vars = []
 
@@ -863,6 +862,11 @@ class CmdVerifier:
             # Lipschitz condition.
             if cur_hp.ghosts:
                 for ghost in cur_hp.ghosts:
+                    # Verify the reasonablity of ghost equations.
+                    # f(x, y) should be in the form: a(x) * y + b(x), y is not in a(x) or b(x)
+                    ghost_var_degree = degree(sympify(str(ghost.diff).replace('^', '**')), gen=symbols(ghost.var))
+                    if not ghost_var_degree in {0,1}:
+                        raise AssertionError("Ghost equations should be linear in ghost variable!")
                     self.infos[pos].eqs_dict[ghost.var] = ghost.diff
                     ghost_vars.append(ghost.var)
                 exists_inv = expr.ExistsExpr(ghost_vars, inv_conj)
@@ -917,7 +921,7 @@ class CmdVerifier:
                         Condition(
                             expr=e, 
                             path=subpost.path,
-                            blabel=subpost.blabel,
+                            blabel=self.compute_branch_label(type=type, cur_branch='execute', post_blabel=subpost.blabel),
                             categ=subpost.categ,
                             origins=subpost.origins + 
                             [OriginLoc(index=i, isInv=True, hp_pos=pos[0]) for i in range(len(cur_hp.inv))] + [OriginLoc(isConstraint=True, hp_pos=pos[0])],
@@ -959,10 +963,11 @@ class CmdVerifier:
 
                     dI_inv = self.infos[sub_pos].inv.expr          
                     # Compute the differential of inv.
-                    # Compute the boundary of constraint. 
-                    # One semi-verification condition is boundary of constraint -> differential of inv.
+                    # One semi-verification condition is closure of constraint -> differential of inv.
+                    # (closure of constraint is not necessary because the differential is always a closed set.)
                     differential = compute_diff(dI_inv, eqs_dict=self.infos[sub_pos].eqs_dict, functions=self.functions)
-                    vc = expr.imp(constraint, differential)
+                    closureD = compute_closed_set(constraint)
+                    vc = expr.imp(closureD, differential)
         
                     self.infos[sub_pos].vcs.append(Condition(expr=vc, 
                                                         path=[sub_pos],
@@ -973,12 +978,12 @@ class CmdVerifier:
                 # Use dbx rule
                 # Cases when dbx_inv is "e == 0".
                     # Use Darboux Equality Rule
-                    #          D -> e_lie_deriv == g * e
+                    #    closure of D -> e_lie_deriv == g * e
                     #--------------------------------------------    (g is the cofactor)
                     #   {e == 0} <x_dot = f(x) & D> {e == 0}
                 # Cases when dbx_inv is e >(>=) 0.
                     # Use Darboux Inequality Rule.
-                    #           D -> e_lie_deriv >= g * e
+                    #     closure of D -> e_lie_deriv >= g * e
                     # ---------------------------------------------
                     #    e >(>=) 0 <x_dot = f(x) & D> e >(>=) 0
                 elif inv.rule == "dbx":
@@ -1008,6 +1013,7 @@ class CmdVerifier:
                         dbx_inv = expr.RelExpr(dbx_inv.op, expr1, expr.AConst(0)) 
                     
                     e = dbx_inv.expr1
+                    assert self.is_polynomial(e, self.constant_names) is True
                     # Compute the lie derivative of e.
                     e_lie_deriv = compute_diff(e, eqs_dict=self.infos[sub_pos].eqs_dict, functions=self.functions)             
 
@@ -1032,8 +1038,10 @@ class CmdVerifier:
                             g = self.infos[sub_pos].dbx_cofactor
                             assert self.is_polynomial(g, self.constant_names) is True
 
-                            # Boundary of D -> e_lie_deriv == g * e
-                            vc = expr.imp(constraint, expr.RelExpr('==', e_lie_deriv, 
+                            # closure of D -> e_lie_deriv == g * e
+                            # (closure of D is not necessary because "e_lie_deriv == g * e" is a closed set)
+                            closureD = compute_closed_set(constraint)
+                            vc = expr.imp(closureD, expr.RelExpr('==', e_lie_deriv, 
                                                                         expr.OpExpr('*', g, e)))
 
                             self.infos[sub_pos].vcs.append(Condition(
@@ -1046,7 +1054,7 @@ class CmdVerifier:
 
                     # Cases when dbx_inv is e >(>=) 0.
                     # Use Darboux Inequality Rule.
-                    #           D -> e_lie_deriv >= g * e
+                    #     closure of D -> e_lie_deriv >= g * e
                     # ---------------------------------------------
                     #    e >(>=) 0 <x_dot = f(x) & D> e >(>=) 0
                     elif dbx_inv.op in {'>', '>='}:
@@ -1062,7 +1070,8 @@ class CmdVerifier:
                                 # remain (e_lie_deriv - g * e) >= 0.
                                 vc_comp = expr.RelExpr('>=', remain, expr.AConst(0))
                                 vc_comps.append(vc_comp)
-                            vc = expr.imp(constraint, expr.list_disj(*vc_comps))
+                            closureD = compute_closed_set(constraint)
+                            vc = expr.imp(closureD, expr.list_disj(*vc_comps))
 
                             self.infos[sub_pos].vcs.append(Condition(
                                                             expr=vc,
@@ -1076,8 +1085,9 @@ class CmdVerifier:
                         else:
                             g = self.infos[sub_pos].dbx_cofactor
                             assert self.is_polynomial(g, self.constant_names) is True
-
-                            vc = expr.imp(constraint, expr.RelExpr('>=', e_lie_deriv, 
+                            
+                            closureD = compute_closed_set(constraint)
+                            vc = expr.imp(closureD, expr.RelExpr('>=', e_lie_deriv, 
                                                                         expr.OpExpr('*', self.infos[sub_pos].dbx_cofactor, e)))
                             self.infos[sub_pos].vcs.append(Condition(
                                                             expr=vc,
@@ -1087,7 +1097,7 @@ class CmdVerifier:
                                                             bottom_loc=bottom_loc, isVC=True))
 
                 # Use barrier certificate
-                #             D && e == 0 -> e_lie > 0
+                #      closure of D && e == 0 -> e_lie > 0
                 # --------------------------------------------------
                 #      {e >=(>) 0} <x_dot = f(x) & D> {e >=(>) 0}                        
                 elif inv.rule == "bc":
@@ -1120,8 +1130,9 @@ class CmdVerifier:
                     e = barrier_inv.expr1
                     e_lie = compute_diff(e, eqs_dict=self.infos[sub_pos].eqs_dict, functions=self.functions)
 
-                    # vc: D && e == 0 -> e_lie > 0
-                    vc = expr.imp(expr.LogicExpr('&&', constraint, 
+                    # vc: closure of D && e == 0 -> e_lie > 0
+                    closureD = compute_closed_set(constraint)
+                    vc = expr.imp(expr.LogicExpr('&&', closureD, 
                                                     expr.RelExpr('==', e, expr.AConst(0))),
                                 expr.RelExpr('>', e_lie, expr.AConst(0)))
 
@@ -1134,14 +1145,21 @@ class CmdVerifier:
                         isVC=True))
 
                 # Use solution axiom
+                #
+                # ---------------------------------------------------------------------------
                 # [[ bar of D, 
                 #    \forall t > 0. (\forall 0 <= s <= t bar of D(y(s))) -> Q(y(t))]] 
                 # <x_dot = f(x) & D(x)> 
                 # [[Q(x)]]
                 #
-                # Assume y(.) solve the symbolic initial value problem y'(t) = f(y), y(0) = x
+                # f(x) is linear in x. Assume y(.) solve the symbolic initial value problem y'(t) = f(y), y(0) = x
                 elif inv.rule == "sln":
                     assert inv.rule_arg is None
+                    for var, diff in self.infos[sub_pos].eqs_dict.items():
+                        d = degree(sympify(str(diff).replace('^', '**')), gen=symbols(var))
+                        if not d in {0,1}:
+                            raise AssertionError("{0} should be linear in {1} when using sln rule!".format(diff, var))
+
                     # Create a new variable to represent time
                     time_var = create_newvar('t', self.names)
 
@@ -1173,7 +1191,7 @@ class CmdVerifier:
                     # Compute the conclusion of implication
                     conclu = Q_y_t
                     
-                    pre = [Condition(expr=expr.ForAllExpr(time_var.name,
+                    pre = pre + [Condition(expr=expr.ForAllExpr(time_var.name,
                                                         expr.imp(expr.RelExpr('>', time_var, expr.AConst(0)),
                                                                 expr.imp(cond, conclu))),
                                     path=[sub_pos],
